/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_BSS_STATS_HXX_
# define WIFI_FRAME_STATS_BSS_STATS_HXX_

# include <set>

# include <boost/foreach.hpp>

# include <wipal/wifi/frame/stats/bss_stats.hh>
# include <wipal/tool/less.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      inline
      bss_stats::bss_stats(): beacon_count_ (0)
      {
      }

      inline
      void
      bss_stats::account_beacon(const addr& bss, const addr& ap)
      {
	single_bss_stats&	s = bss_stats_[bss];

	++beacon_count_;
	++s.total_;
	++s.ap_stats_[ap];
      }

      inline
      std::ostream&
      bss_stats::print(std::ostream& o) const
      {
	typedef std::set<internal_stats::const_iterator,
			 tool::less_using_second>		sorted_type;

	sorted_type	sorted;

	for (internal_stats::const_iterator i = bss_stats_.begin();
	     i != bss_stats_.end();
	     ++i)
	  sorted.insert(i);

	o << "  total beacon count: " << beacon_count_ << '\n'
	  << "  distinct BSSIDs: " << bss_stats_.size() << '\n';

	BOOST_FOREACH(const internal_stats::const_iterator& i, sorted)
	  {
	    typedef single_bss_stats::ap_stats::value_type     single_ap_stats;

	    const single_bss_stats::ap_stats&	s = i->second.ap_stats_;

	    o << '\n'
	      << "  begin BSS: " << i->first << '\n'
	      << "    beacon count: " << i->second.total_ << '\n'
	      << "    beacon ratio: "
	      <<   (double (i->second.total_) / beacon_count_) << '\n'
	      << "    distinct beacon senders: " << s.size() << '\n';

	    assert(not s.empty());
	    if (s.size() > 1 or s.begin()->first != i->first)
	      {
		BOOST_FOREACH(const single_ap_stats& p, i->second.ap_stats_)
		  o << '\n'
		    << "    begin beacon sender: (" << i->first << ") "
		    <<                                p.first << '\n'
		    << "      beacon count: " << p.second << '\n'
		    << "      beacon count ratio (total): "
		    <<          (double (p.second) / beacon_count_) << '\n'
		    << "      beacon count ratio (this BSS): "
		    <<          (double (p.second) / i->second.total_) << '\n'
		    << "    end beacon sender: (" << i->first << ") "
		    <<                              p.first << '\n';
	      }
	    o << "  end BSS: " << i->first << '\n';
	  }
	return o << std::flush;
      }

      inline
      bss_stats::single_bss_stats::single_bss_stats(): total_ (0)
      {
      }

      inline
      bool
      bss_stats::
      single_bss_stats::operator < (const single_bss_stats& rhs) const
      {
	return total_ < rhs.total_;
      }

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_STATS_BSS_STATS_HXX_
