/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_GROWTH_HXX_
# define WIFI_FRAME_STATS_GROWTH_HXX_

# include <wipal/wifi/frame/stats/growth.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      template <class T, unsigned EL>
      void
      growth<T, EL>::restart()
      {
	tracker_.restart();
	elements_.clear();
	current_elements_.clear();
      }

      template <class T, unsigned EL>
      void
      growth<T, EL>::update(const tool::microseconds& timestamp)
      {
	if (tracker_.update(timestamp))
	  {
	    epoch_data&	d = tracker_.current();

	    current_elements_.clear();
	    d.total_count = 0;
	    d.current_count = 0;
	  }
      }

      template <class T, unsigned EL>
      void
      growth<T, EL>::update(const T& elt)
      {
	epoch_data&	d = tracker_.current();

	if (current_elements_.insert(elt).second)
	  ++d.current_count;

	if (elements_.insert(elt).second)
	  ++d.total_count;
      }

      template <class T, unsigned EL>
      std::ostream&
      growth<T, EL>::print(std::ostream& o) const
      {
	unsigned	total = 0;

	BOOST_FOREACH(const epoch_data& e, tracker_.epochs())
	  {
	    total += e.total_count;
	    o << '\t' << e.total_count
	      << '\t' << total
	      << '\t' << e.current_count << '\n';
	  }
	return o << std::flush;
      }

      template <class T, unsigned EL>
      growth<T, EL>::epoch_data::epoch_data(): total_count (0),
					       current_count (0)
      {
      }

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_STATS_GROWTH_HXX_
