/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_SIMPLE_COUNTERS_HH_
# define WIFI_FRAME_STATS_SIMPLE_COUNTERS_HH_

extern "C"
{
# include <stdint.h>
}

# include <wipal/wifi/frame/stats/abstract_stats.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      struct simple_counters: public abstract_stats
      {
	simple_counters();

	void		account_frame(unsigned	len,
				      unsigned	type,
				      unsigned	subtype,
				      bool	retransmitted,
				      bool	from_ap);
	void		account_gap();
	void		account_expired();
	void		account_out_of_order();
	void		account_duplicate();
	void		account_seq_anomaly();
	std::ostream&	print(std::ostream& o) const;

	unsigned	total() const;
	unsigned	type_count(unsigned type) const ;
	unsigned	subtype_count(unsigned type, unsigned subtype) const;

      private:
	unsigned	total_;
	uint64_t	size_;
	unsigned	ap_total_;
	uint64_t	ap_size_;
	unsigned	retransmitted_;
	unsigned	expired_;
	unsigned	out_of_order_;
	unsigned	duplicate_;
	unsigned	gap_;
	unsigned	seq_anomaly_;
	unsigned	type_count_[4];
	uint64_t	type_size_[4];
	unsigned	subtype_count_[4][16];
	uint64_t	subtype_size_[4][16];
      };

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "simple_counters.hxx"

#endif // ! WIFI_FRAME_STATS_SIMPLE_COUNTERS_HH_
