/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_TFI_PLOT_HH_
# define WIFI_FRAME_STATS_TFI_PLOT_HH_

# include <boost/multi_array.hpp>

# include <wipal/wifi/frame/stats/abstract_stats.hh>
# include <wipal/tool/microseconds.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      struct tfi_plot: public abstract_stats
      {
	tfi_plot();

	void		account_frame(unsigned			frame_id,
				      const tool::microseconds&	arrival);
	void		account_gap(unsigned gap_length);
	void		account_miss();

	void		restart();

	std::ostream&	print(std::ostream& o) const;

      private:
	void		end_epoch(const tool::microseconds& new_epoch_start);

	enum
	  {
	    epoch_length	= 102400,
	    score_step		= 10,
	    load_step		= 1,
	    load_max		= 2048
	  };

	boost::multi_array<unsigned, 2>	tfi_;
	tool::microseconds		epoch_start_;
	unsigned			total_;
	unsigned			missed_;
      };

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "tfi_plot.hxx"

#endif // ! WIFI_FRAME_STATS_TFI_PLOT_HH_
