/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_HASH_HXX_
# define WIFI_FRAME_HASH_HXX_

# include <wipal/wifi/frame/unique_id/hash.hh>
# include <wipal/wifi/frame/frame.hh>
# include <wipal/wifi/frame/mgt.hh>

namespace wifi
{

  namespace frame
  {

    namespace internals
    {

      hash_value::hash_value(const void* data, size_t len)
      {
	MD5(static_cast<const unsigned char*> (data), len, &this->front());
      }

      struct hash_hooks: public dissector_default_hooks
      {

	inline
	void
	probe_resp_hook(const mgt::header* frame, size_t caplen)
	{
	  if (not control_flag(3, frame))
	    result_ = hash_value (frame, caplen);
	}

	inline
	void
	beacon_hook(const mgt::header* frame, size_t caplen)
	{
	  result_ = hash_value (frame, caplen);
	}

	boost::optional<hash_value>	result_;
      };

    } // End of namespace wifi::frame::internals.

    template <class HeaderType, class D>
    boost::optional<hsh_id>
    unique_id_factory<hsh_id>::build(const pcapxx::frame_descriptor<D>& desc,
				     addr_mapping&)
    {
      WP_UNIQUE_ID_DISSECT(desc, d, internals::hash_hooks, h);

      if (h.result_)
	return hsh_id (desc.id(), h.result_.get());
      else
	return boost::none_t ();
    }

  } // End of namespace wifi::frame.

} // End of namespace wifi.

WP_HASH_NAMESPACE_BEGIN

  inline
  size_t
  hash<wifi::frame::internals::hash_value>::
  operator () (const wifi::frame::internals::hash_value& v) const
  {
    typedef wifi::frame::internals::hash_value	hash_value;
    typedef hash_value::value_type		value_type;
    typedef hash_value::const_iterator		const_iterator;

    size_t			r = 0;
    const_iterator		i = v.begin();
    const const_iterator	e = v.end();

    while (i != e)
      for (unsigned j = 0; i != e and j < sizeof (size_t); ++i, ++j)
	r ^= (*i << (j * 8));

    return r;
  }

WP_HASH_NAMESPACE_END

#endif // ! WIFI_FRAME_HASH_HXX_
